@@ -137,12 +137,23 @@ def parse_args():
         help=('Load only emoji which were added to Unicode '
               + 'not earlier than this Unicode version. '
               + 'default: %(default)s'))
+    # Guess current Unicode version:
+    unicode_versions = (
+        ('20180605', '11.0'),
+        ('20190701', '12.0'),
+        ('20200701', '13.0'),
+    )
+    current_date = time.strftime('%Y%m%d')
+    current_unicode_version = 10.0
+    for (date, version) in unicode_versions:
+        if current_date > date:
+            current_unicode_version = version
     parser.add_argument(
         '--emoji_unicode_max',
         nargs='?',
         type=str,
         action='store',
-        default='100.0',
+        default=current_unicode_version,
         help=('Load only emoji which were added to Unicode '
               + 'not later than this Unicode version. '
               + 'default: %(default)s'))
@@ -1450,7 +1461,8 @@ class EmojiPickerUI(Gtk.Window):
             rectangle.width = self._fontsize * 1.5
             rectangle.height = self._fontsize * 1.5
             self._emoji_selected_popover.set_pointing_to(rectangle)
-        label = Gtk.Label(_('Copied to clipboard!'))
+        label = Gtk.Label()
+        label.set_text(_('Copied to clipboard!'))
         self._emoji_selected_popover.add(label)
         if GTK_VERSION >= (3, 22, 0):
             self._emoji_selected_popover.popup()
@@ -1572,7 +1584,8 @@ class EmojiPickerUI(Gtk.Window):
         self._skin_tone_selected_popover.set_relative_to(
             flowbox_child.get_child())
         self._skin_tone_selected_popover.set_position(Gtk.PositionType.TOP)
-        label = Gtk.Label(_('Copied to clipboard!'))
+        label = Gtk.Label()
+        label.set_text(_('Copied to clipboard!'))
         self._skin_tone_selected_popover.add(label)
         if GTK_VERSION >= (3, 22, 0):
             self._skin_tone_selected_popover.popup()
@@ -2054,7 +2067,8 @@ class EmojiPickerUI(Gtk.Window):
             if filter_text.replace(' ', '').lower()
             in font.replace(' ', '').lower()]
         for font in fonts:
-            label = Gtk.Label(font)
+            label = Gtk.Label()
+            label.set_text(font)
             label.set_xalign(0)
             margin = 1
             label.set_margin_start(margin)
